@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.6
 # -*- coding: utf-8 -*-
 #
 # (c) Copyright 2003-2008 Hewlett-Packard Development Company, L.P.
@@ -98,6 +98,9 @@ TECH_CLASSES = [
     "Corbett",
     "Python",
     "OJ7000",
+    "Pyramid",
+    "Python10",
+    "Mimas",
 ]
 
 TECH_CLASSES.sort()
@@ -148,6 +151,8 @@ TECH_CLASS_PDLS = {
     "Corbett"       : 'pcl3',
     "Python"        : 'pcl3',
     "OJ7000"        : 'pcl3',
+    "Python10"      : 'pcl3',
+    "Mimas"      : 'pcl3',
 }
 
 PDL_TYPE_PCL = 0  # less preferred
@@ -196,6 +201,10 @@ TECH_SUBCLASSES = [
     "NoAutoDuplex",
     "NoCDDVD",
     "NoMaxDPI",
+    "NoMaxDPI",
+    "SmallMargins",
+    "Trim",
+    "4800x1200dpi",
 ]
 
 TECH_SUBCLASSES.sort()
