@@ -22,15 +22,27 @@
 from __future__ import division
 
 # Std Lib
-import struct, cStringIO 
+import struct
+import cStringIO
 import xml.parsers.expat as expat
 import re
+import urllib
+try:
+    from xml.etree import ElementTree
+    etree_loaded = True
+except ImportError:
+    try:
+        from elementtree.ElementTree import XML
+        elementtree_loaded = True
+    except ImportError:
+        elementtree_loaded = False
+    etree_loaded = False
 
 # Local
 from g import *
 from codes import *
 import pml, utils
-
+import hpmudext
 
 """
 status dict structure:
@@ -126,96 +138,114 @@ STATUS_POS = {STATUS_REV_00 : 14,
                STATUS_REV_04 : 20}
 
 def parseSStatus(s, z=''):
+    revision = ''
+    pens = []
+    top_door = TOP_DOOR_NOT_PRESENT
+    stat = STATUS_UNKNOWN
+    supply_door = SUPPLY_DOOR_NOT_PRESENT
+    duplexer = DUPLEXER_NOT_PRESENT
+    photo_tray = PHOTO_TRAY_NOT_PRESENT
+    in_tray1 = IN_TRAY_NOT_PRESENT
+    in_tray2 = IN_TRAY_NOT_PRESENT
+    media_path = MEDIA_PATH_NOT_PRESENT
     Z_SIZE = 6
 
-    z1 = []
-    if len(z) > 0:
-        z_fields = z.split(',')
+    try:
+        z1 = []
+        if len(z) > 0:
+            z_fields = z.split(',')
 
-        for z_field in z_fields:
+            for z_field in z_fields:
 
-            if len(z_field) > 2 and z_field[:2] == '05':
-                z1s = z_field[2:]
-                z1 = [int(x, 16) for x in z1s]
+                if len(z_field) > 2 and z_field[:2] == '05':
+                    z1s = z_field[2:]
+                    z1 = [int(x, 16) for x in z1s]
 
-    s1 = [int(x, 16) for x in s]
+        s1 = [int(x, 16) for x in s]
 
-    revision = s1[1]
+        revision = s1[1]
 
-    assert STATUS_REV_00 <= revision <= STATUS_REV_04
+        assert STATUS_REV_00 <= revision <= STATUS_REV_04
 
-    top_door = bool(s1[2] & 0x8L) + s1[2] & 0x1L
-    supply_door = bool(s1[3] & 0x8L) + s1[3] & 0x1L
-    duplexer = bool(s1[4] & 0xcL) +  s1[4] & 0x1L
-    photo_tray = bool(s1[5] & 0x8L) + s1[5] & 0x1L
-
-    if revision == STATUS_REV_02:
-        in_tray1 = bool(s1[6] & 0x8L) + s1[6] & 0x1L
-        in_tray2 = bool(s1[7] & 0x8L) + s1[7] & 0x1L
-    else:
-        in_tray1 = bool(s1[6] & 0x8L)
-        in_tray2 = bool(s1[7] & 0x8L)
-
-    media_path = bool(s1[8] & 0x8L) + (s1[8] & 0x1L) + ((bool(s1[18] & 0x2L))<<1)
-    status_pos = STATUS_POS[revision]
-    status_byte = (s1[status_pos]<<4) + s1[status_pos + 1]
-    stat = status_byte + STATUS_PRINTER_BASE
-
-    pens, pen, c, d = [], {}, NUM_PEN_POS[revision]+1, 0
-    num_pens = s1[NUM_PEN_POS[revision]]
-    log.debug("Num pens=%d" % num_pens)
-    index = 0
-    pen_data_size = PEN_DATA_SIZE[revision]
-
-    for p in range(num_pens):
-        info = long(s[c : c + pen_data_size], 16)
-
-        pen['index'] = index
-
-        if pen_data_size == 4:
-            pen['type'] = REVISION_2_TYPE_MAP.get(int((info & 0xf000L) >> 12L), 0)
-
-            if index < (num_pens / 2):
-                pen['kind'] = AGENT_KIND_HEAD
-            else:
-                pen['kind'] = AGENT_KIND_SUPPLY
-
-            pen['level-trigger'] = int ((info & 0x0e00L) >> 9L)
-            pen['health'] = int((info & 0x0180L) >> 7L)
-            pen['level'] = int(info & 0x007fL)
-            pen['id'] = 0x1f
-
-        elif pen_data_size == 8:
-            pen['kind'] = bool(info & 0x80000000L) + ((bool(info & 0x40000000L))<<1L)
-            pen['type'] = int((info & 0x3f000000L) >> 24L)
-            pen['id'] = int((info & 0xf80000) >> 19L)
-            pen['level-trigger'] = int((info & 0x70000L) >> 16L)
-            pen['health'] = int((info & 0xc000L) >> 14L)
-            pen['level'] = int(info & 0xffL)
+        top_door = bool(s1[2] & 0x8L) + s1[2] & 0x1L
+        supply_door = bool(s1[3] & 0x8L) + s1[3] & 0x1L
+        duplexer = bool(s1[4] & 0xcL) +  s1[4] & 0x1L
+        photo_tray = bool(s1[5] & 0x8L) + s1[5] & 0x1L
 
+        if revision == STATUS_REV_02:
+            in_tray1 = bool(s1[6] & 0x8L) + s1[6] & 0x1L
+            in_tray2 = bool(s1[7] & 0x8L) + s1[7] & 0x1L
         else:
-            log.error("Pen data size error")
+            in_tray1 = bool(s1[6] & 0x8L)
+            in_tray2 = bool(s1[7] & 0x8L)
 
-        if len(z1) > 0:
-            # TODO: Determine cause of IndexError for C6100 (defect #1111)
-            try:
-                pen['dvc'] = long(z1s[d+1:d+5], 16)
-                pen['virgin'] = bool(z1[d+5] & 0x8L)
-                pen['hp-ink'] = bool(z1[d+5] & 0x4L)
-                pen['known'] = bool(z1[d+5] & 0x2L)
-                pen['ack'] = bool(z1[d+5] & 0x1L)
-            except IndexError:
-                pen['dvc'] = 0
-                pen['virgin'] = 0
-                pen['hp-ink'] = 0
-                pen['known'] = 0
-                pen['ack'] = 0
+        media_path = bool(s1[8] & 0x8L) + (s1[8] & 0x1L) + ((bool(s1[18] & 0x2L))<<1)
+        status_pos = STATUS_POS[revision]
+        status_byte = s1[status_pos]<<4
+        if status_byte != 48:
+            status_byte = (s1[status_pos]<<4) + s1[status_pos + 1]
+        stat = status_byte + STATUS_PRINTER_BASE
 
-        index += 1
-        pens.append(pen)
-        pen = {}
-        c += pen_data_size
-        d += Z_SIZE
+        pen, c, d = {}, NUM_PEN_POS[revision]+1, 0
+        num_pens = s1[NUM_PEN_POS[revision]]
+        index = 0
+        pen_data_size = PEN_DATA_SIZE[revision]
+
+        log.debug("num_pens = %d" % num_pens)
+        for p in range(num_pens):
+            info = long(s[c : c + pen_data_size], 16)
+
+            pen['index'] = index
+
+            if pen_data_size == 4:
+                pen['type'] = REVISION_2_TYPE_MAP.get(int((info & 0xf000L) >> 12L), 0)
+
+                if index < (num_pens / 2):
+                    pen['kind'] = AGENT_KIND_HEAD
+                else:
+                    pen['kind'] = AGENT_KIND_SUPPLY
+
+                pen['level-trigger'] = int ((info & 0x0e00L) >> 9L)
+                pen['health'] = int((info & 0x0180L) >> 7L)
+                pen['level'] = int(info & 0x007fL)
+                pen['id'] = 0x1f
+
+            elif pen_data_size == 8:
+                pen['kind'] = bool(info & 0x80000000L) + ((bool(info & 0x40000000L))<<1L)
+                pen['type'] = int((info & 0x3f000000L) >> 24L)
+                pen['id'] = int((info & 0xf80000) >> 19L)
+                pen['level-trigger'] = int((info & 0x70000L) >> 16L)
+                pen['health'] = int((info & 0xc000L) >> 14L)
+                pen['level'] = int(info & 0xffL)
+
+            else:
+                log.error("Pen data size error")
+
+            if len(z1) > 0:
+                # TODO: Determine cause of IndexError for C6100 (defect #1111)
+                try:
+                    pen['dvc'] = long(z1s[d+1:d+5], 16)
+                    pen['virgin'] = bool(z1[d+5] & 0x8L)
+                    pen['hp-ink'] = bool(z1[d+5] & 0x4L)
+                    pen['known'] = bool(z1[d+5] & 0x2L)
+                    pen['ack'] = bool(z1[d+5] & 0x1L)
+                except IndexError:
+                    pen['dvc'] = 0
+                    pen['virgin'] = 0
+                    pen['hp-ink'] = 0
+                    pen['known'] = 0
+                    pen['ack'] = 0
+
+            log.debug("pen %d %s" % (index, pen))
+
+            index += 1
+            pens.append(pen)
+            pen = {}
+            c += pen_data_size
+            d += Z_SIZE
+
+    except (IndexError, ValueError, TypeError), e:
+        log.warn("Status parsing error: %s" % str(e))
 
     return {'revision' :    revision,
              'agents' :      pens,
@@ -236,6 +266,7 @@ def parseSStatus(s, z=''):
 def parseVStatus(s):
     pens, pen, c = [], {}, 0
     fields = s.split(',')
+    log.debug(fields)
     f0 = fields[0]
 
     if len(f0) == 20:
@@ -284,18 +315,26 @@ def parseVStatus(s):
     else:
         pass
 
-    if fields[2] == 'DN':
-        top_lid = 1
+    try:
+        fields[2]
+    except IndexError:
+        top_lid = 1 # something went wrong!
     else:
-        top_lid = 2
+        if fields[2] == 'DN':
+            top_lid = 1
+        else:
+            top_lid = 2
 
-    stat = vstatus_xlate.get(fields[3].lower(), STATUS_PRINTER_IDLE)
+    try:
+        stat = vstatus_xlate.get(fields[3].lower(), STATUS_PRINTER_IDLE)
+    except IndexError:
+        stat = STATUS_PRINTER_IDLE # something went wrong!
 
     return {'revision' :   STATUS_REV_V,
              'agents' :     pens,
-             'top-lid' :    top_lid,
+             'top-door' :   top_lid,
              'status-code': stat,
-             'supply-lid' : SUPPLY_DOOR_NOT_PRESENT,
+             'supply-door': SUPPLY_DOOR_NOT_PRESENT,
              'duplexer' :   DUPLEXER_NOT_PRESENT,
              'photo-tray' : PHOTO_TRAY_NOT_PRESENT,
              'in-tray1' :   IN_TRAY_NOT_PRESENT,
@@ -303,6 +342,7 @@ def parseVStatus(s):
              'media-path' : MEDIA_PATH_CUT_SHEET, # ?
            }
 
+
 def parseStatus(DeviceID):
     if 'VSTATUS' in DeviceID:
          return parseVStatus(DeviceID['VSTATUS'])
@@ -311,8 +351,6 @@ def parseStatus(DeviceID):
     else:
         return STATUS_BLOCK_UNKNOWN
 
-
-
 def LaserJetDeviceStatusToPrinterStatus(device_status, printer_status, detected_error_state):
     stat = STATUS_PRINTER_IDLE
 
@@ -642,32 +680,33 @@ setup_panel_translator()
 
 def PanelCheck(dev):
     line1, line2 = '', ''
-    try:
-        dev.openPML()
-    except Error:
-        pass
-    else:
 
-        oids = [(pml.OID_HP_LINE1, pml.OID_HP_LINE2),
-                 (pml.OID_SPM_LINE1, pml.OID_SPM_LINE2)]
+    if dev.io_mode not in (IO_MODE_RAW, IO_MODE_UNI):
 
-        for oid1, oid2 in oids:
-            result, line1 = dev.getPML(oid1)
+        try:
+            dev.openPML()
+        except Error:
+            pass
+        else:
 
-            if result < pml.ERROR_MAX_OK:
-                line1 = PANEL_TRANSLATOR_FUNC(line1).rstrip()
+            oids = [(pml.OID_HP_LINE1, pml.OID_HP_LINE2),
+                     (pml.OID_SPM_LINE1, pml.OID_SPM_LINE2)]
 
-                if '\x0a' in line1:
-                    line1, line2 = line1.split('\x0a', 1)
-                    break
-
-                result, line2 = dev.getPML(oid2)
+            for oid1, oid2 in oids:
+                result, line1 = dev.getPML(oid1)
 
                 if result < pml.ERROR_MAX_OK:
-                    line2 = PANEL_TRANSLATOR_FUNC(line2).rstrip()
-                    break
+                    line1 = PANEL_TRANSLATOR_FUNC(line1).rstrip()
 
-        #dev.closePML()
+                    if '\x0a' in line1:
+                        line1, line2 = line1.split('\x0a', 1)
+                        break
+
+                    result, line2 = dev.getPML(oid2)
+
+                    if result < pml.ERROR_MAX_OK:
+                        line2 = PANEL_TRANSLATOR_FUNC(line2).rstrip()
+                        break
 
     return bool(line1 or line2), line1 or '', line2 or ''
 
@@ -700,62 +739,92 @@ BATTERY_PML_TRIGGER_MAP = {
         }
 
 
-def BatteryCheck(dev, status_block):
+def BatteryCheck(dev, status_block, battery_check):
     try_dynamic_counters = False
 
     try:
         try:
             dev.openPML()
         except Error:
-            log.debug("PML channel open failed. Trying dynamic counters...")
-            try_dynamic_counters = True
+            if battery_check == STATUS_BATTERY_CHECK_STD:
+                log.debug("PML channel open failed. Trying dynamic counters...")
+                try_dynamic_counters = True
         else:
-            result, battery_level = dev.getPML(pml.OID_BATTERY_LEVEL)
-            result, power_mode =  dev.getPML(pml.OID_POWER_MODE)
-
-            if battery_level is not None and \
-                power_mode is not None:
-
-                if power_mode & pml.POWER_MODE_BATTERY_LEVEL_KNOWN and \
-                    battery_level >= 0:
-
-                    for x in BATTERY_PML_TRIGGER_MAP:
-                        if x[0] >= battery_level > x[1]:
-                            battery_trigger_level = BATTERY_PML_TRIGGER_MAP[x]
-                            break
-
-                    if power_mode & pml.POWER_MODE_CHARGING:
-                        agent_health = AGENT_HEALTH_CHARGING
-
-                    elif power_mode & pml.POWER_MODE_DISCHARGING:
-                        agent_health = AGENT_HEALTH_DISCHARGING
-
-                    else:
-                        agent_health = AGENT_HEALTH_OK
+            if battery_check == STATUS_BATTERY_CHECK_PML:
+                result, battery_level = dev.getPML(pml.OID_BATTERY_LEVEL_2)
 
+                if result > pml.ERROR_MAX_OK:
                     status_block['agents'].append({
-                                                    'kind'   : AGENT_KIND_INT_BATTERY,
-                                                    'type'   : AGENT_TYPE_UNSPECIFIED,
-                                                    'health' : agent_health,
-                                                    'level'  : battery_level,
-                                                    'level-trigger' : battery_trigger_level,
-                                                    })
+                        'kind'   : AGENT_KIND_INT_BATTERY,
+                        'type'   : AGENT_TYPE_UNSPECIFIED,
+                        'health' : AGENT_HEALTH_UNKNOWN,
+                        'level'  : 0,
+                        'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+                        })
+                    return
+
                 else:
                     status_block['agents'].append({
-                                                    'kind'   : AGENT_KIND_INT_BATTERY,
-                                                    'type'   : AGENT_TYPE_UNSPECIFIED,
-                                                    'health' : AGENT_HEALTH_UNKNOWN,
-                                                    'level'  : 0,
-                                                    'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
-                                                    })
+                        'kind'   : AGENT_KIND_INT_BATTERY,
+                        'type'   : AGENT_TYPE_UNSPECIFIED,
+                        'health' : AGENT_HEALTH_OK,
+                        'level'  : battery_level,
+                        'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+                        })
+                    return
 
-            else:
-                try_dynamic_counters = True
+            else: # STATUS_BATTERY_CHECK_STD
+                result, battery_level = dev.getPML(pml.OID_BATTERY_LEVEL)
+                result, power_mode =  dev.getPML(pml.OID_POWER_MODE)
+
+                if battery_level is not None and \
+                    power_mode is not None:
+
+                    if power_mode & pml.POWER_MODE_BATTERY_LEVEL_KNOWN and \
+                        battery_level >= 0:
+
+                        for x in BATTERY_PML_TRIGGER_MAP:
+                            if x[0] >= battery_level > x[1]:
+                                battery_trigger_level = BATTERY_PML_TRIGGER_MAP[x]
+                                break
+
+                        if power_mode & pml.POWER_MODE_CHARGING:
+                            agent_health = AGENT_HEALTH_CHARGING
+
+                        elif power_mode & pml.POWER_MODE_DISCHARGING:
+                            agent_health = AGENT_HEALTH_DISCHARGING
+
+                        else:
+                            agent_health = AGENT_HEALTH_OK
+
+                        status_block['agents'].append({
+                            'kind'   : AGENT_KIND_INT_BATTERY,
+                            'type'   : AGENT_TYPE_UNSPECIFIED,
+                            'health' : agent_health,
+                            'level'  : battery_level,
+                            'level-trigger' : battery_trigger_level,
+                            })
+                        return
+
+                    else:
+                        status_block['agents'].append({
+                            'kind'   : AGENT_KIND_INT_BATTERY,
+                            'type'   : AGENT_TYPE_UNSPECIFIED,
+                            'health' : AGENT_HEALTH_UNKNOWN,
+                            'level'  : 0,
+                            'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+                            })
+                        return
+
+                else:
+                    try_dynamic_counters = True
 
     finally:
         dev.closePML()
 
-    if try_dynamic_counters:
+
+    if battery_check == STATUS_BATTERY_CHECK_STD and \
+        try_dynamic_counters:
 
         try:
             try:
@@ -764,23 +833,33 @@ def BatteryCheck(dev, status_block):
                 battery_level = dev.getDynamicCounter(202)
 
                 status_block['agents'].append({
-                                                'kind'   : AGENT_KIND_INT_BATTERY,
-                                                'type'   : AGENT_TYPE_UNSPECIFIED,
-                                                'health' : BATTERY_HEALTH_MAP[battery_health],
-                                                'level'  : battery_level,
-                                                'level-trigger' : BATTERY_TRIGGER_MAP[battery_trigger_level],
-                                                })
+                    'kind'   : AGENT_KIND_INT_BATTERY,
+                    'type'   : AGENT_TYPE_UNSPECIFIED,
+                    'health' : BATTERY_HEALTH_MAP[battery_health],
+                    'level'  : battery_level,
+                    'level-trigger' : BATTERY_TRIGGER_MAP[battery_trigger_level],
+                    })
             except Error:
                 status_block['agents'].append({
-                                                'kind'   : AGENT_KIND_INT_BATTERY,
-                                                'type'   : AGENT_TYPE_UNSPECIFIED,
-                                                'health' : AGENT_HEALTH_UNKNOWN,
-                                                'level'  : 0,
-                                                'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
-                                                })
+                    'kind'   : AGENT_KIND_INT_BATTERY,
+                    'type'   : AGENT_TYPE_UNSPECIFIED,
+                    'health' : AGENT_HEALTH_UNKNOWN,
+                    'level'  : 0,
+                    'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+                    })
         finally:
             dev.closePrint()
 
+    else:
+        status_block['agents'].append({
+            'kind'   : AGENT_KIND_INT_BATTERY,
+            'type'   : AGENT_TYPE_UNSPECIFIED,
+            'health' : AGENT_HEALTH_UNKNOWN,
+            'level'  : 0,
+            'level-trigger' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+            })
+
+
 
 # this works for 2 pen products that allow 1 or 2 pens inserted
 # from: k, kcm, cmy, ggk
@@ -824,23 +903,24 @@ def getPenConfiguration(s): # s=status dict from parsed device ID
 def getFaxStatus(dev):
     tx_active, rx_active = False, False
 
-    try:
-        dev.openPML()
+    if dev.io_mode not in (IO_MODE_UNI, IO_MODE_RAW):
+        try:
+            dev.openPML()
 
-        result_code, tx_state = dev.getPML(pml.OID_FAXJOB_TX_STATUS)
+            result_code, tx_state = dev.getPML(pml.OID_FAXJOB_TX_STATUS)
 
-        if result_code == ERROR_SUCCESS and tx_state:
-            if tx_state not in (pml.FAXJOB_TX_STATUS_IDLE, pml.FAXJOB_TX_STATUS_DONE):
-                tx_active = True
+            if result_code == ERROR_SUCCESS and tx_state:
+                if tx_state not in (pml.FAXJOB_TX_STATUS_IDLE, pml.FAXJOB_TX_STATUS_DONE):
+                    tx_active = True
 
-        result_code, rx_state = dev.getPML(pml.OID_FAXJOB_RX_STATUS)
+            result_code, rx_state = dev.getPML(pml.OID_FAXJOB_RX_STATUS)
 
-        if result_code == ERROR_SUCCESS and rx_state:
-            if rx_state not in (pml.FAXJOB_RX_STATUS_IDLE, pml.FAXJOB_RX_STATUS_DONE):
-                rx_active = True
+            if result_code == ERROR_SUCCESS and rx_state:
+                if rx_state not in (pml.FAXJOB_RX_STATUS_IDLE, pml.FAXJOB_RX_STATUS_DONE):
+                    rx_active = True
 
-    finally:
-        dev.closePML()
+        finally:
+            dev.closePML()
 
     return tx_active, rx_active
 
@@ -862,7 +942,7 @@ TYPE6_STATUS_CODE_MAP = {
     -19730: STATUS_PRINTER_OUT_OF_PAPER,
     -19729: STATUS_PRINTER_OUT_OF_PAPER,
     -19933: STATUS_PRINTER_HARD_ERROR, # out of memory
-    -17984: STATUS_PRINTER_DOOR_OPEN, 
+    -17984: STATUS_PRINTER_DOOR_OPEN,
     -19694: STATUS_PRINTER_DOOR_OPEN,
     -18992: STATUS_PRINTER_MANUAL_FEED_BLOCKED, # ?
     -19690: STATUS_PRINTER_MEDIA_JAM, # tray 1
@@ -966,14 +1046,17 @@ TYPE6_STATUS_CODE_MAP = {
     -13848: STATUS_PRINTER_BUSY, #</DevStatusFaxMemoryFullReceive>
     -13849: STATUS_PRINTER_BUSY, #</DevStatusFaxReceiveError>
 
-}    
+}
 
 def StatusType6(dev): #  LaserJet Status (XML)
     info_device_status = cStringIO.StringIO()
     info_ssp = cStringIO.StringIO()
 
-    dev.getEWSUrl("/hp/device/info_device_status.xml", info_device_status)
-    dev.getEWSUrl("/hp/device/info_ssp.xml", info_ssp)
+    try:
+        dev.getEWSUrl("/hp/device/info_device_status.xml", info_device_status)
+        dev.getEWSUrl("/hp/device/info_ssp.xml", info_ssp)
+    except:
+        pass
 
     info_device_status = info_device_status.getvalue()
     info_ssp = info_ssp.getvalue()
@@ -983,8 +1066,8 @@ def StatusType6(dev): #  LaserJet Status (XML)
 
     if info_device_status:
         try:
-            device_status = utils.XMLToDictParser().parseXML(info_device_status)
             log.debug_block("info_device_status", info_device_status)
+            device_status = utils.XMLToDictParser().parseXML(info_device_status)
             log.debug(device_status)
         except expat.ExpatError:
             log.error("Device Status XML parse error")
@@ -992,11 +1075,11 @@ def StatusType6(dev): #  LaserJet Status (XML)
 
     if info_ssp:
         try:
-            ssp = utils.XMLToDictParser().parseXML(info_ssp)
             log.debug_block("info_spp", info_ssp)
+            ssp = utils.XMLToDictParser().parseXML(info_ssp)
             log.debug(ssp)
         except expat.ExpatError:
-            log.error("SSP XML parse error")            
+            log.error("SSP XML parse error")
             ssp = {}
 
     status_code = device_status.get('devicestatuspage-devicestatus-statuslist-status-code-0', 0)
@@ -1050,7 +1133,7 @@ def StatusType6(dev): #  LaserJet Status (XML)
              'in-tray2' :    1,
              'media-path' :  1,
              'status-code' : TYPE6_STATUS_CODE_MAP.get(status_code, STATUS_PRINTER_IDLE),
-           }     
+           }
 
 # PJL status codes
 PJL_STATUS_MAP = {
@@ -1161,7 +1244,7 @@ def MapPJLErrorCode(error_code, str_code=None):
 
     if str_code is None:
         str_code = str(error_code)
-        
+
     if len(str_code) < 5:
         return STATUS_PRINTER_BUSY
 
@@ -1237,50 +1320,63 @@ def MapPJLErrorCode(error_code, str_code=None):
 
 pjl_code_pat = re.compile("""^CODE\s*=\s*(\d.*)$""", re.IGNORECASE)
 
-def StatusType8(dev): #  LaserJet PJL
+
+
+def StatusType8(dev): #  LaserJet PJL (B&W only)
     try:
+        # Will error if printer is busy printing...
         dev.openPrint()
     except Error, e:
         log.warn(e.msg)
+        status_code = STATUS_PRINTER_BUSY
+    else:
+        try:
+            try:
+                dev.writePrint("\x1b%-12345X@PJL INFO STATUS \r\n\x1b%-12345X")
+                pjl_return = dev.readPrint(1024, timeout=5, allow_short_read=True)
+                dev.close()
 
-    dev.writePrint("\x1b%-12345X@PJL INFO STATUS \r\n\x1b%-12345X")
-    pjl_return = dev.readPrint(1024, timeout=5, allow_short_read=True)
-    dev.close()
+                log.debug_block("PJL return:", pjl_return)
 
-    log.debug_block("PJL return:", pjl_return)
+                str_code = '10001'
 
-    str_code = '10001'
+                for line in pjl_return.splitlines():
+                    line = line.strip()
+                    match = pjl_code_pat.match(line)
 
-    for line in pjl_return.splitlines():
-        line = line.strip()
-        match = pjl_code_pat.match(line)
+                    if match is not None:
+                        str_code = match.group(1)
+                        break
 
-        if match is not None:
-            str_code = match.group(1)
-            break
+                log.debug("Code = %s" % str_code)
 
-    log.debug("Code = %s" % str_code)
+                try:
+                    error_code = int(str_code)
+                except ValueError:
+                    error_code = DEFAULT_PJL_ERROR_CODE
 
-    try:
-        error_code = int(str_code)
-    except ValueError:
-        error_code = DEFAULT_PJL_ERROR_CODE
+                log.debug("Error code = %d" % error_code)
 
-    log.debug("Error code = %d" % error_code)
-
-    status_code = MapPJLErrorCode(error_code, str_code)
+                status_code = MapPJLErrorCode(error_code, str_code)
+            except Error:
+                status_code = STATUS_PRINTER_HARD_ERROR
+        finally:
+            try:
+                dev.closePrint()
+            except Error:
+                pass
 
     agents = []
 
     # TODO: Only handles mono lasers...
     if status_code in (STATUS_PRINTER_LOW_TONER, STATUS_PRINTER_LOW_BLACK_TONER):
         health = AGENT_HEALTH_OK
-        level_trigger = AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT
+        level_trigger = AGENT_LEVEL_TRIGGER_MAY_BE_LOW
         level = 0
 
     elif status_code == STATUS_PRINTER_NO_TONER:
         health = AGENT_HEALTH_MISINSTALLED
-        level_trigger = AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT
+        level_trigger = AGENT_LEVEL_TRIGGER_MAY_BE_LOW
         level = 0
 
     else:
@@ -1290,7 +1386,6 @@ def StatusType8(dev): #  LaserJet PJL
 
     log.debug("Agent: health=%d, level=%d, trigger=%d" % (health, level, level_trigger))
 
-
     agents.append({  'kind' : AGENT_KIND_TONER_CARTRIDGE,
                      'type' : AGENT_TYPE_BLACK,
                      'health' : health,
@@ -1298,6 +1393,52 @@ def StatusType8(dev): #  LaserJet PJL
                      'level-trigger' : level_trigger,
                   })
 
+    if dev.tech_type == TECH_TYPE_COLOR_LASER:
+        level = 100
+        level_trigger = AGENT_LEVEL_TRIGGER_SUFFICIENT_0
+        if status_code == STATUS_PRINTER_LOW_CYAN_TONER:
+            level = 0
+            level_trigger = AGENT_LEVEL_TRIGGER_MAY_BE_LOW
+
+        log.debug("Agent: health=%d, level=%d, trigger=%d" % (health, level, level_trigger))
+
+        agents.append({  'kind' : AGENT_KIND_TONER_CARTRIDGE,
+                         'type' : AGENT_TYPE_CYAN,
+                         'health' : AGENT_HEALTH_OK,
+                         'level' : level,
+                         'level-trigger' : level_trigger,
+                      })
+
+        level = 100
+        level_trigger = AGENT_LEVEL_TRIGGER_SUFFICIENT_0
+        if status_code == STATUS_PRINTER_LOW_MAGENTA_TONER:
+            level = 0
+            level_trigger = AGENT_LEVEL_TRIGGER_MAY_BE_LOW
+
+        log.debug("Agent: health=%d, level=%d, trigger=%d" % (health, level, level_trigger))
+
+        agents.append({  'kind' : AGENT_KIND_TONER_CARTRIDGE,
+                         'type' : AGENT_TYPE_MAGENTA,
+                         'health' : AGENT_HEALTH_OK,
+                         'level' : level,
+                         'level-trigger' : level_trigger,
+                      })
+
+        level = 100
+        level_trigger = AGENT_LEVEL_TRIGGER_SUFFICIENT_0
+        if status_code == STATUS_PRINTER_LOW_YELLOW_TONER:
+            level = 0
+            level_trigger = AGENT_LEVEL_TRIGGER_MAY_BE_LOW
+
+        log.debug("Agent: health=%d, level=%d, trigger=%d" % (health, level, level_trigger))
+
+        agents.append({  'kind' : AGENT_KIND_TONER_CARTRIDGE,
+                         'type' : AGENT_TYPE_YELLOW,
+                         'health' : AGENT_HEALTH_OK,
+                         'level' : level,
+                         'level-trigger' : level_trigger,
+                      })
+
     if status_code == 40021:
         top_door = 0
     else:
@@ -1315,8 +1456,230 @@ def StatusType8(dev): #  LaserJet PJL
              'in-tray2' :    1,
              'media-path' :  1,
              'status-code' : status_code,
-           }     
+           }
 
 
+element_type10_xlate = { 'ink' : AGENT_KIND_SUPPLY,
+                         'inkCartridge' : AGENT_KIND_HEAD_AND_SUPPLY,
+                         'printhead' : AGENT_KIND_HEAD,
+                         'toner' : AGENT_KIND_TONER_CARTRIDGE,
+                       }
+
+pen_type10_xlate = { 'pK' : AGENT_TYPE_PG,
+                     'CMY' : AGENT_TYPE_CMY,
+                     'M' : AGENT_TYPE_MAGENTA,
+                     'C' : AGENT_TYPE_CYAN,
+                     'Y' : AGENT_TYPE_YELLOW,
+                     'K' : AGENT_TYPE_BLACK,
+                   }
+
+pen_level10_xlate = { 'ok' : AGENT_LEVEL_TRIGGER_SUFFICIENT_0,
+                      'low' : AGENT_LEVEL_TRIGGER_MAY_BE_LOW,
+                      'out' : AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT,
+                      'empty' : AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT,
+                      'missing' : AGENT_LEVEL_TRIGGER_ALMOST_DEFINITELY_OUT,
+                    }
+
+pen_health10_xlate = { 'ok' : AGENT_HEALTH_OK,
+                       'misinstalled' : AGENT_HEALTH_MISINSTALLED,
+                       'missing' : AGENT_HEALTH_MISINSTALLED,
+                     }
+
+def StatusType10FetchUrl(dev, url):
+    if dev.is_local:
+       data_fp = cStringIO.StringIO()
+       dev.getEWSUrl_LEDM(url, data_fp)
+       data = data_fp.getvalue()
+
+    else:
+        if dev.zc:
+            status, ip = hpmudext.get_zc_ip_address(dev.zc)
+            if status != hpmudext.HPMUD_R_OK:
+                log.error("unable to get IP address of mDNS configured device")
+                return None
+        else:
+            ip = dev.host
+
+        # Get the agent status XML
+        addr = "http://%s:8080%s" % (ip, url)
+        feed = urllib.urlopen(addr)
+        data = feed.read()
+        feed.close()
+
+    return data
 
 
+def StatusType10(dev): # Low End Data Model
+    status_block = { 'revision' :    STATUS_REV_UNKNOWN,
+                     'agents' :      [],
+                     'top-door' :    TOP_DOOR_NOT_PRESENT,
+                     'supply-door' : TOP_DOOR_NOT_PRESENT,
+                     'duplexer' :    DUPLEXER_NOT_PRESENT,
+                     'photo-tray' :  PHOTO_TRAY_NOT_PRESENT,
+                     'in-tray1' :    IN_TRAY_NOT_PRESENT,
+                     'in-tray2' :    IN_TRAY_NOT_PRESENT,
+                     'media-path' :  MEDIA_PATH_NOT_PRESENT,
+                     'status-code' : STATUS_PRINTER_IDLE,
+                   }
+
+    if not etree_loaded and not elementtree_loaded:
+        log.error("cannot get status for printer. please load ElementTree module")
+        return status_block
+
+    # Get the dynamic consumables configuration
+    data = StatusType10FetchUrl(dev, "/DevMgmt/ConsumableConfigDyn.xml")
+    if not data:
+        return status_block
+    data = data.replace("ccdyn:", "").replace("dd:", "")
+
+    # Parse the agent status XML
+    agents = []
+    try:
+        if etree_loaded:
+            tree = ElementTree.XML(data)
+        if not etree_loaded and elementtree_loaded:
+            tree = XML(data)
+        elements = tree.findall("ConsumableInfo")
+        for e in elements:
+            health = AGENT_HEALTH_OK
+            ink_level = 0
+            type = e.find("ConsumableTypeEnum").text
+            state = e.find("ConsumableLifeState/ConsumableState").text
+
+            # level
+            if type == "ink" or type == "inkCartridge" or type == "toner":
+                ink_type = e.find("ConsumableLabelCode").text
+                if state != "missing":
+                    try:
+                       ink_level = int(e.find("ConsumablePercentageLevelRemaining").text)
+                    except:
+                       ink_level = 0
+            else:
+                ink_type = ''
+                if state == "ok":
+                    ink_level = 100
+
+            log.debug("type '%s' state '%s' ink_type '%s' ink_level %d" % (type, state, ink_type, ink_level))
+    
+            entry = { 'kind' : element_type10_xlate.get(type, AGENT_KIND_NONE),
+                      'type' : pen_type10_xlate.get(ink_type, AGENT_TYPE_NONE),
+                      'health' : pen_health10_xlate.get(state, AGENT_HEALTH_OK),
+                      'level' : int(ink_level),
+                      'level-trigger' : pen_level10_xlate.get(state, AGENT_LEVEL_TRIGGER_SUFFICIENT_0)
+                    }
+
+            log.debug("%s" % entry)
+            agents.append(entry)
+    except (expat.ExpatError, UnboundLocalError):
+        agents = []
+    status_block['agents'] = agents
+
+    # Get the media handling configuration
+    data = StatusType10FetchUrl(dev, "/DevMgmt/MediaHandlingDyn.xml")
+    if not data:
+        return status_block
+    data = data.replace("mhdyn:", "").replace("dd:", "")
+
+    # Parse the media handling XML
+    try:
+        if etree_loaded:
+            tree = ElementTree.XML(data)
+        if not etree_loaded and elementtree_loaded:
+            tree = XML(data)
+        elements = tree.findall("InputTray")
+    except (expat.ExpatError, UnboundLocalError):
+        elements = []
+    for e in elements:
+        bin_name = e.find("InputBin").text
+        if bin_name == "Tray1":
+            status_block['in-tray1'] = IN_TRAY_PRESENT
+        elif bin_name == "Tray2":
+            status_block['in-tray2'] = IN_TRAY_PRESENT
+        elif bin_name == "PhotoTray":
+            status_block['photo-tray'] = PHOTO_TRAY_ENGAGED
+        else:
+            log.error("found invalid bin name '%s'" % bin_name)
+
+    try:
+        elements = tree.findall("Accessories/MediaHandlingDeviceFunctionType")
+    except UnboundLocalError:
+        elements = []
+    for e in elements:
+        if e.text == "autoDuplexor":
+            status_block['duplexer'] = DUPLEXER_DOOR_CLOSED
+
+    # Get the product status
+    data = StatusType10FetchUrl(dev, "/DevMgmt/ProductStatusDyn.xml")
+    if not data:
+        return status_block
+    data = data.replace("psdyn:", "").replace("locid:", "")
+    data = data.replace("pscat:", "").replace("dd:", "").replace("ad:", "")
+
+    # Parse the product status XML
+    try:
+        if etree_loaded:
+            tree = ElementTree.XML(data)
+        if not etree_loaded and elementtree_loaded:
+            tree = XML(data)
+        elements = tree.findall("Status/StatusCategory")
+    except (expat.ExpatError, UnboundLocalError):
+        elements = []
+    for e in elements:
+        if e.text == "closeDoorOrCover":
+            status_block['status-code'] = STATUS_PRINTER_DOOR_OPEN
+        elif e.text == "shuttingDown":
+            status_block['status-code'] = STATUS_PRINTER_TURNING_OFF
+        elif e.text == "cancelJob":
+            status_block['status-code'] = STATUS_PRINTER_CANCELING
+        elif e.text == "trayEmptyOrOpen":
+            status_block['status-code'] = STATUS_PRINTER_OUT_OF_PAPER
+        elif e.text == "jamInPrinter":
+            status_block['status-code'] = STATUS_PRINTER_MEDIA_JAM
+        elif e.text == "hardError":
+            status_block['status-code'] = STATUS_PRINTER_HARD_ERROR
+        elif e.text == "outputBinFull":
+            status_block['status-code'] = STATUS_PRINTER_OUTPUT_BIN_FULL
+        elif e.text == "unexpectedSizeInTray":
+            status_block['status-code'] = STATUS_PRINTER_MEDIA_SIZE_MISMATCH
+        elif e.text == "insertOrCloseTray2":
+            status_block['status-code'] = STATUS_PRINTER_TRAY_2_MISSING
+        elif e.text == "scannerError":
+            status_block['status-code'] = EVENT_SCANNER_FAIL
+        elif e.text == "scanProcessing":
+            status_block['status-code'] = EVENT_START_SCAN_JOB
+        elif e.text == "scannerAdfLoaded":
+            status_block['status-code'] = EVENT_SCAN_ADF_LOADED
+        elif e.text == "scanToDestinationNotSet":
+            status_block['status-code'] = EVENT_SCAN_TO_DESTINATION_NOTSET
+        elif e.text == "scanWaitingForPC":
+            status_block['status-code'] = EVENT_SCAN_WAITING_FOR_PC
+        elif e.text == "scannerAdfJam":
+            status_block['status-code'] = EVENT_SCAN_ADF_JAM
+        elif e.text == "scannerAdfDoorOpen":
+            status_block['status-code'] = EVENT_SCAN_ADF_DOOR_OPEN
+        elif e.text == "faxProcessing":
+            status_block['status-code'] = EVENT_START_FAX_JOB
+        elif e.text == "faxSending":
+            status_block['status-code'] = STATUS_FAX_TX_ACTIVE
+        elif e.text == "faxReceiving":
+            status_block['status-code'] = STATUS_FAX_RX_ACTIVE
+        elif e.text == "faxDialing":
+            status_block['status-code'] = EVENT_FAX_DIALING
+        elif e.text == "faxConnecting":
+            status_block['status-code'] = EVENT_FAX_CONNECTING
+        elif e.text == "faxSendError":
+            status_block['status-code'] = EVENT_FAX_SEND_ERROR
+        elif e.text == "faxErrorStorageFull":
+            status_block['status-code'] = EVENT_FAX_ERROR_STORAGE_FULL
+        elif e.text == "faxReceiveError":
+            status_block['status-code'] = EVENT_FAX_RECV_ERROR
+        elif e.text == "faxBlocking":
+            status_block['status-code'] = EVENT_FAX_BLOCKING
+        elif e.text == "inPowerSave":
+            status_block['status-code'] = STATUS_PRINTER_POWER_SAVE
+        elif e.text == "incorrectCartridge":
+            status_block['status-code'] = STATUS_PRINTER_CARTRIDGE_WRONG
+        elif e.text == "cartridgeMissing":
+            status_block['status-code'] = STATUS_PRINTER_CARTRIDGE_MISSING
+
+    return status_block
\ No newline at end of file
