@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 #
-# Author: Don Welch
+# Author: Don Welch, Naga Samrat Chowdary Narla
 #
 # Thanks to Henrique M. Holschuh <hmh@debian.org> for various security patches
 #
@@ -26,6 +26,7 @@ from __future__ import generators
 # Std Lib
 import sys
 import os
+import grp
 import fnmatch
 import tempfile
 import socket
@@ -58,8 +59,35 @@ import pexpect
 BIG_ENDIAN = 0
 LITTLE_ENDIAN = 1
 
+def addgroup():
+    lis = []
+    try:
+        fp=open('/etc/cups/cupsd.conf')
+    except IOError:
+        try:
+            if "root" != grp.getgrgid(os.stat('/etc/cups/cupsd.conf').st_gid)[0]:
+                return grp.getgrgid(os.stat('/etc/cups/cupsd.conf').st_gid)[0]
+        except OSError:
+            return lis
 
+    try:
+        lis = ((re.findall('SystemGroup [\w* ]*',fp.read()))[0].replace('SystemGroup ','')).split(' ')
+    except IndexError:
+        return lis
 
+    if 'root' in lis:
+        lis.remove('root')
+    fp.close()
+    return lis
+
+def list_to_string(lis):
+    if len(lis) == 0:
+        return ""
+    if len(lis) == 1:
+        return str("\""+lis[0]+"\"")
+    if len(lis) >= 1:
+        return "\""+"\", \"".join(lis)+"\" and \""+str(lis.pop())+"\""
+        
 def lock(f):
     log.debug("Locking: %s" % f.name)
     try:
@@ -469,7 +497,7 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         if len(path):
             self.cmd_pcard = 'hp-unload -d %DEVICE_URI%'
         else:
-            self.cmd_pcard = '/usr/local/bin/python2.5 %HOME%/unload.py -d %DEVICE_URI%'
+            self.cmd_pcard = '/usr/local/bin/python2.6 %HOME%/unload.py -d %DEVICE_URI%'
 
         # Copy
         path = which('hp-makecopies')
@@ -477,7 +505,7 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         if len(path):
             self.cmd_copy = 'hp-makecopies -d %DEVICE_URI%'
         else:
-            self.cmd_copy = '/usr/local/bin/python2.5 %HOME%/makecopies.py -d %DEVICE_URI%'
+            self.cmd_copy = '/usr/local/bin/python2.6 %HOME%/makecopies.py -d %DEVICE_URI%'
 
         # Fax
         path = which('hp-sendfax')
@@ -485,7 +513,7 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         if len(path):
             self.cmd_fax = 'hp-sendfax -d %FAX_URI%'
         else:
-            self.cmd_fax = '/usr/local/bin/python2.5 %HOME%/sendfax.py -d %FAX_URI%'
+            self.cmd_fax = '/usr/local/bin/python2.6 %HOME%/sendfax.py -d %FAX_URI%'
 
         # Fax Address Book
         path = which('hp-fab')
@@ -493,7 +521,7 @@ class UserSettings(object): # Note: Deprecated after 2.8.8 in Qt4 (see ui4/ui_ut
         if len(path):
             self.cmd_fab = 'hp-fab'
         else:
-            self.cmd_fab = '/usr/local/bin/python2.5 %HOME%/fab.py'
+            self.cmd_fab = '/usr/local/bin/python2.6 %HOME%/fab.py'
 
     def load(self):
         self.loadDefaults()
@@ -812,8 +840,8 @@ def all(S,f=lambda x:x):
     return True
 
 
-BROWSERS = ['firefox', 'mozilla', 'konqueror', 'galeon', 'skipstone'] # in preferred order
-BROWSER_OPTS = {'firefox': '-new-window', 'mozilla' : '', 'konqueror': '', 'galeon': '-w', 'skipstone': ''}
+BROWSERS = ['xdg-open']
+BROWSER_OPTS = {'xdg-open' : ''}
 
 
 def find_browser():
