@@ -1483,7 +1483,6 @@ class CoreInstall(object):
                             yield d, self.dependencies[d][2], opt
                             # depend, desc, option
 
-
     def missing_optional_dependencies(self):
         # missing deps in opt. options
         for opt in self.components[self.selected_component][1]:
@@ -1501,10 +1500,17 @@ class CoreInstall(object):
                 for d in self.options[opt][2]: # dependencies for option
                     if d == 'cups-ddk':
                         status, output = self.run('cups-config --version')
-                        if status == 0:
-  			    import string
-                            major, minor, release = string.split(output, '.', 3)
-                            if major > 1 or (major == 1 and minor >= 4):
+                        import string
+                        if status == 0 and (string.count(output, '.') == 1 or string.count(output, '.') == 2):
+                            if string.count(output, '.') == 1:
+                                major, minor = string.split(output, '.', 2)
+                            if string.count(output, '.') == 2:
+                                major, minor, release = string.split(output, '.', 3)
+                            if len(minor) > 1 and minor[1] >= '0' and minor[1] <= '9':
+                                minor = ((ord(minor[0]) - ord('0')) * 10) + (ord(minor[1]) - ord('0'))
+                            else:
+                                minor = ord(minor[0]) - ord('0')
+                            if major > '1' or (major == '1' and minor >= 4):
                                 continue
 	            if not self.dependencies[d][0]: # optional dep
                         if not self.have_dependencies[d]: # missing
@@ -1512,7 +1518,6 @@ class CoreInstall(object):
                             yield d, self.dependencies[d][2], self.dependencies[d][0], opt
                             # depend, desc, option
 
-
     def select_options(self, answer_callback):
         num_opt_missing = 0
         # not-required options
@@ -1648,7 +1653,7 @@ class CoreInstall(object):
     def pre_build(self):
         cmds = []
         if self.get_distro_ver_data('fix_ppd_symlink', False):
-            cmds.append(self.su_sudo() % '/usr/local/bin/python2.5 ./installer/fix_symlink.py')
+            cmds.append(self.su_sudo() % '/usr/local/bin/python2.6 ./installer/fix_symlink.py')
 
         return cmds
 
@@ -1989,7 +1994,11 @@ class CoreInstall(object):
         if mode == GUI_MODE:
             return os.system("sh %s --nox11 -- -u" % plugin_file) == 0
         else:
-            return os.system("sh %s --nox11 -- -i" % plugin_file) == 0
+            if os.system("sh %s --nox11 -- -i" % plugin_file) == 0:
+                return True
+            else:
+                log.error("Python gobject/dbus may be not installed")
+                return False
 
 
     def delete_plugin(self):
